/*----------------------------------------------------------------*
 * SUGI PAPER 326-2010
 * Adding Statistical Functionality to the DATA Step with FCMP
 *
 * BY: Stacey Christian and Jacques Rioux
 *
 * EXAMPLE 2: Iterated Reweighted Least Squares Algorithm 
 * 
 * Data creation for exmple 2,
 *---------------------------------------------------------------*/
/******************************************************************/
/* Macro to create a sample simulated data set:                   */
/* The base survival model is Exponential with mean=10            */
/* There are 3 categorical covariates: X1, X2 and X3              */
/*         X1 = 0 or 1                                            */
/*         X2 = 0 or 0.4 or 0.9                                   */
/*         X3 = -1 or 1                                           */
/* The model simulated corresponds to 0.1*X1 + 0.3*X2 + 0.2*X3    */
/* The variables in the dataset are X1, X2, X3,Quarter, and Count */
/******************************************************************/
%macro generate_survival_data_macro;
 %let dsname = %sysfunc(dequote(&dsname));
 DATA &dsname;
   array AX1[2] _temporary_ (0 1);
   array AX2[3] _temporary_ (0 0.4 0.9);
   array AX3[2] _temporary_ (-1 1);
   array ATime_[13] Time_0-Time_12;

   do i = 1  to dim(AX1);
      do j = 1 to dim(AX2);
         do k = 1 to dim(AX3);
	   X1       = AX1[i];
	   X2       = AX2[j];
	   X3       = AX3[k];
	   Effect = exp(0.1 * X1 + 0.3 * X2 + 0.2 * X3);

	   do l = 1 to 13;
	      ATime_[l] = 0;
	   end;

	   do l = 1 to 1000;
	      Time = 10/Effect * rand("Exponential");
	      Quarters = ceil(4*time);

	      do m = 1 to 13;
		 if Quarters > m-1 then do;
		    ATime_[m] = ATime_[m] + 1;
		 end;
	      end;
	   end;

	   lastSurv = 1;

	   do l = 1 to 13;
	      Quarter = l - 1;
	      Count = ATime_[l];
		    Surv = 1 - cdf("Exponential",(l-1)/40);
		    Prob = Surv/lastSurv;
		    lastSurv = Surv;
	      output;
	   end;
         end;
      end;
   end;

   drop Effect Surv Prob;
   drop i j k l m  quarters lastSurv time time_:;
 RUN;
%mend generate_survival_data_macro;

/* run macro and create data for Example 2 */
%let dsname= mdphData;
%generate_survival_data_macro;
