/*----------------------------------------------------------------*
 * SUGI PAPER 326-2010
 * Adding Statistical Functionality to the DATA Step with FCMP
 *
 * BY: Stacey Christian and Jacques Rioux
 *
 * EXAMPLE 3: PH Model Simulation
 *
 * NOTE: code in thos file depends on having run 
 *    MinDistData.sas and MinDistFit.sas first.
 *---------------------------------------------------------------*/

%macro append_data_macro;
   %let data = %sysfunc( dequote(&data) );
   %let base = %sysfunc( dequote(&base) );
   
   proc append base = &base 
           data = &data 
           force;
   quit;
%mend append_data_macro;

proc fcmp outlib=sasuser.example2.sim ;
    subroutine simulate_phdata( dsname $);
        rc = run_macro('generate_survival_data_macro', dsname);
    endsub;

    subroutine append_data( base $, data $ );
        rc = run_macro('append_data_macro', base, data );
    endsub;

    subroutine fit_ph_model(data $, parmDataOut $,
                            countVar $, timeVar $, depVar $, weightVar $,
                            regressors $);
      /*- initial data preparation -*/
      length intervalVars $ 128;
      intervalCount = 0; intervalVars = "";
      call prepare_phdata( data, "Count", "Quarter", regressors, "_prepdat_", 
                           intervalCount, intervalVars );
      independentVars = regressors || " " || intervalVars;

      /*- create param arrays dynamically */
      numParams = intervalCount  + count(regressors, " ")+1;
      array params1[1] _temporary_;
      array params2[1] _temporary_;
      call dynamic_array(params1, numParams );
      call dynamic_array(params2, numParams );

      /*- initial regression run with initial weights -*/
      call run_regression( "_prepdat_", depVar, independentVars, 
                           weightVar, parmDataOut, params1 );

      /*- iterative loop -*/
      maxRelativeDifference = 1;
      do while( maxRelativeDifference > 0.0001 ); 
         call update_weights("_prepdat_", parmDataOut, regressors, weightVar, intervalCount);
            
         call run_regression( "_prepdat_", depVar, independentVars,
                              weightVar, parmDataOut, params2 );  
 
         maxRelativeDifference = calc_max_relative_diff(params1,params2);
         put maxRelativeDifference=;
      end;

      call update_weights("_prepdat_", parmDataOut, regressors, weightVar, intervalCount);
    endsub;
run;              


options cmplib=(sasuser.example2);

%let NSIM=20;
proc fcmp;
	do i=1 to &NSIM;
	    put "*******************";
	    put "Simulation" i;
	    put "*******************";

        call simulate_phdata("work.simdata");
	    call fit_ph_model("work.simdata", "work.params",
                          "Count", "Quarter", "log_log_Pij", "Weight",
		                  "x1 x2 x3");				  
	    call append_data("work.simresults", "work.params");
	end;
run;   

proc summary data= work.simresults;
    var X1 X2 X3 Interval_1-Interval_12;
    output
        out = work.simsummary
        mean = meanX1 meanX2 meanX3 meanInterval_1-meanInterval_12
        var  = varX1 varX2 varX3 varInterval_1-varInterval_12
        ;
run;

