/*--------------------------------------------------------------*
 * SUGI PAPER 326-2010
 * Adding Statistical Functionality to the DATA Step with FCMP
 *
 * BY: Stacey Christian and Jacques Rioux
 *
 * EXAMPLE 1: Recursive algorithm with PROC REG to segment 
 *            time series data
 * 
 * Code for plotting piecewise linear approximations versus
 * original data.
 *-------------------------------------------------------------*/


/*---------------------------------------------------*
 * Macro to plot the piecewise linear approximations  
 * versus original data.  
 *
 * Parameters:
 *    seg_ds   - name of segment dataset
 *    var      - variable that was modeled
 *    interval - interval of data
 *    prefix   - ds name prefix for regression data
 *---------------------------------------------------*/
%macro plot_segments_macro;

  %let seg_ds   = %sysfunc(dequote(&seg_ds));
  %let prefix   = %sysfunc(dequote(&prefix));
  %let var      = %sysfunc(dequote(&var));
  %let interval = %sysfunc(dequote(&interval));

  /* combined segments in one dataset */
  %let nobs=0;
  %let ds=&seg_ds;
  %get_nobs_macro;

  %do i=1 %to &nobs;
    data _NULL_; set &seg_ds(firstobs=&i obs=&i);
	  length dsname $ 256;
	  dsname = trim(left("&prefix")) || trim(left(start)) || "_" || trim(left(end));
      call symputx("ds&i", trim(dsname));
    run;
  %end;

  data _combined_; set &ds1; run;
  %do i=2 %to &nobs;
    proc append base= _combined_ data = &&ds&i; run;
  %end;
  data _combined_; set _combined_; label PREDICT="Linear Approximation of &var"; run;

  ods graphics on; ods html;
  proc sgplot data=_combined_;
    series x=date y=&var / lineattrs=(pattern=solid thickness=0.1);
    series x=date y=PREDICT / lineattrs=(pattern=solid thickness=1);
  run;
  ods graphics off; ods html close;

%mend plot_segments_macro;

proc fcmp outlib=sasuser.examples.topdown;
  subroutine plot_segments(seg_ds $, var $, interval $, prefix $);
      rc = run_macro('plot_segments_macro', seg_ds, var, interval, prefix);
  endsub;
run;

options cmplib=(sasuser.examples) nomprint;
data _NULL_;
 * call plot_segments("segds_reg_15","close","daily","_reg_");
  call plot_segments("segds_reg_20","close","daily","_reg_");
run;

